{
  Demo:
  How to extract text from RTF and DocX files without using visual components.

  https://www.trichview.com

}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, RVRTF, RVRTFErr, RVDocXReader, Menus, StdCtrls, RVTypes, RVUni;

type

  TCustomRVMSWordReaderClass = class of TCustomRVMSWordReader;

  TForm1 = class(TForm)
    Memo1: TMemo;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Open1: TMenuItem;
    OpenDialog1: TOpenDialog;
    procedure Open1Click(Sender: TObject);
  private
    { Private declarations }
    FirstLine: Boolean;
    procedure DoReaderText(Sender: TCustomRVMSWordReader; const Text: TRVAnsiString;
      Position: TRVRTFPosition);
    procedure DoReaderTextW(Sender: TCustomRVMSWordReader; const Text: TRVUnicodeString;
      Position: TRVRTFPosition);
    function ExtractTextFromFile(const FileName: TRVUnicodeString;
      ReaderClass: TCustomRVMSWordReaderClass): Boolean;
  public
    { Public declarations }
    ResultText: TRVUnicodeString;
    function ExtractTextFromRTF(const FileName: TRVUnicodeString): Boolean;
    function ExtractTextFromDocX(const FileName: TRVUnicodeString): Boolean;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

{ TForm1 }

function TForm1.ExtractTextFromFile(const FileName: TRVUnicodeString;
  ReaderClass: TCustomRVMSWordReaderClass): Boolean;
var
  parser: TCustomRVMSWordReader;
begin
  ResultText := '';
  FirstLine := True;
  parser := ReaderClass.Create(nil);
  try
    if parser is TRVRTFReader then
      TRVRTFReader(parser).OnNewText := DoReaderText;
    parser.OnNewUnicodeText := DoReaderTextW;
    Result := parser.ReadFromFile(FileName);
  finally
    parser.Free;
  end;
end;

function TForm1.ExtractTextFromRTF(const FileName: TRVUnicodeString): Boolean;
begin
  Result := ExtractTextFromFile(FileName, TRVRTFReader);
end;

function TForm1.ExtractTextFromDocX(const FileName: TRVUnicodeString): Boolean;
begin
  Result := ExtractTextFromFile(FileName, TRVDocXReader);
end;

procedure TForm1.DoReaderText(Sender: TCustomRVMSWordReader; const Text: TRVAnsiString;
  Position: TRVRTFPosition);
var
  TextW: TRVUnicodeString;
  CodePage: Cardinal;
begin
  if (Position<>rtf_ts_ContinuePara) and not FirstLine then
    ResultText := ResultText + #13#10;
  FirstLine := False;
  if TRVRTFReader(Sender).FontTable.Count=0 then
    CodePage := CP_ACP
  else
    CodePage := RVU_Charset2CodePage(
      TRVRTFReader(Sender).FontTable[TRVRTFReader(Sender).RTFState.CharProps.FontIndex].Charset);
   TextW := RVU_AnsiToUnicode(CodePage, Text);
  ResultText := ResultText + TextW;
end;

procedure TForm1.DoReaderTextW(Sender: TCustomRVMSWordReader;
  const Text: TRVUnicodeString; Position: TRVRTFPosition);
begin
  if (Position<>rtf_ts_ContinuePara) and not FirstLine then
    ResultText := ResultText + #13#10;
  FirstLine := False;
  ResultText := ResultText + Text;
end;

procedure TForm1.Open1Click(Sender: TObject);
var
  Res: Boolean;
begin
  if OpenDialog1.Execute then
  begin
    if CompareText(ExtractFileExt(OpenDialog1.FileName), '.docx') = 0 then
      Res := ExtractTextFromDocX(RVU_UIStringToUnicode(OpenDialog1.FileName))
    else
      Res := ExtractTextFromRTF(RVU_UIStringToUnicode(OpenDialog1.FileName));
    if Res then
      Memo1.Lines.Text := ResultText
    else
      ShowMessage('Error loading file');
  end;
end;



end.
